@@ -25,7 +25,7 @@
 
 #include "upsclient.h"
 
-static void help(char *prog)
+static void help(const char *prog)
 {
 	printf("Network UPS Tools upsc %s\n\n", UPS_VERSION);
 
@@ -37,7 +37,7 @@ static void help(char *prog)
 	printf("  <variable> - optional, display this variable only.\n");
 	printf("               Default: list all variables for <host>\n");
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 static void clean_exit(UPSCONN *ups, char *upsname, char *hostname, int code)
@@ -53,33 +53,22 @@ static void clean_exit(UPSCONN *ups, char *upsname, char *hostname, int code)
 	exit(code);
 }
 
-static int old_printvar(UPSCONN *ups, const char *upsname, const char *var)
+static int printvar(UPSCONN *ups, const char *upsname, const char *var)
 {
-	char	value[SMALLBUF];
-
-	if (upscli_getvar(ups, upsname, var, value, sizeof(value)) < 0) {
-		fprintf(stderr, "Error: %s\n", upscli_strerror(ups));
-		return 1;
-	}
-
-	printf("%s\n", value);
-
-	return 0;
-}
-
-static int new_printvar(UPSCONN *ups, const char *upsname, const char *var)
-{
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
 	/* old-style variable name? */
-	if (!strchr(var, '.'))
-		return old_printvar(ups, upsname, var);
+	if (!strchr(var, '.')) {
+		fprintf(stderr, "Error: old-style variable names are not supported\n");
+		return EXIT_FAILURE;
+	}
 
 	if (!upsname) {
 		fprintf(stderr, "Error: ups name must be defined\n");
-		return 1;
+		return EXIT_FAILURE;
 	}
 
 	query[0] = "VAR";
@@ -95,70 +84,34 @@ static int new_printvar(UPSCONN *ups, const char *upsname, const char *var)
 		/* new var and old upsd?  try to explain the situation */
 		if (upscli_upserror(ups) == UPSCLI_ERR_UNKCOMMAND) {
 			fprintf(stderr, "Error: variable unknown (old upsd detected)\n");
-			return 1;
+			return EXIT_FAILURE;
 		}
 
 		fprintf(stderr, "Error: %s\n", upscli_strerror(ups));
-		return 1;
+		return EXIT_FAILURE;
 	}
 
 	if (numa < numq) {
 		fprintf(stderr, "Error: insufficient data "
 			"(got %d args, need at least %d)\n", numa, numq);
-		return 1;
+		return EXIT_FAILURE;
 	}
 
 	printf("%s\n", answer[3]);
-	return 0;
+	return EXIT_SUCCESS;
 }
 
-static int old_list(UPSCONN *ups, const char *upsname)
+static int list_vars(UPSCONN *ups, const char *upsname)
 {
-	char	vars[LARGEBUF], *v, *ptr;
-
-	if (upscli_getlist(ups, upsname, UPSCLI_LIST_VARS, vars, 
-		sizeof(vars)) < 0) {
-		fprintf(stderr, "Unable to get variable list - %s\n", 
-		        upscli_strerror(ups));
-
-		return 1;
-	}
-
-	if (upsname)
-		printf("UPS: %s\n", upsname);
-	else
-		printf("UPS: (default)\n");
-
-	if (strlen(vars) == 0) {
-		fprintf(stderr, "No data available - check your "
-			"configuration (ups.conf)\n");
-
-		return 1;
-	}
-
-	v = vars;
-	while (v != NULL) {
-		ptr = strchr(v, ' ');
-		if (ptr)
-			*ptr++ = '\0';
-
-		printf("%s: ", v);
-		old_printvar(ups, upsname, v);
-
-		v = ptr;
-	}		
-
-	return 0;
-}
-
-static int new_list(UPSCONN *ups, const char *upsname)
-{
-	int	ret, numq, numa;
+	int	ret;
+	unsigned int	numq, numa;
 	const	char	*query[4];
 	char	**answer;
 
-	if (!upsname)
-		return old_list(ups, upsname);
+	if (!upsname) {
+		fprintf(stderr, "Error: a UPS name must be specified (upsname@hostname)\n");
+		return EXIT_FAILURE;
+	}
 
 	query[0] = "VAR";
 	query[1] = upsname;
@@ -168,12 +121,14 @@ static int new_list(UPSCONN *ups, const char *upsname)
 
 	if (ret < 0) {
 
-		/* old upsd --> fall back on old LISTVARS technique */
-		if (upscli_upserror(ups) == UPSCLI_ERR_UNKCOMMAND)
-			return old_list(ups, upsname);
+		/* check for an old upsd */
+		if (upscli_upserror(ups) == UPSCLI_ERR_UNKCOMMAND) {
+			fprintf(stderr, "Error: upsd is too old to support this query\n");
+			return EXIT_FAILURE;
+		}
 
 		fprintf(stderr, "Error: %s\n", upscli_strerror(ups));
-		return 1;
+		return EXIT_FAILURE;
 	}
 
 	ret = upscli_list_next(ups, numq, query, &numa, &answer);
@@ -184,7 +139,7 @@ static int new_list(UPSCONN *ups, const char *upsname)
 		if (numa < 4) {
 			fprintf(stderr, "Error: insufficient data "
 				"(got %d args, need at least 4)\n", numa);
-			return 1;
+			return EXIT_FAILURE;
 		}
 
 		printf("%s: %s\n", answer[2], answer[3]);
@@ -192,7 +147,21 @@ static int new_list(UPSCONN *ups, const char *upsname)
 		ret = upscli_list_next(ups, numq, query, &numa, &answer);
 	}
 
-	return 0;
+	return EXIT_SUCCESS;
+}
+
+static void check_upsdef(const char *ups)
+{
+	char	*ptr;
+
+	ptr = strchr(ups, '@');
+
+	if (ptr)
+		return;
+
+	fprintf(stderr, "Error: invalid UPS definition.  Required format: upsname@hostname[:port]\n");
+
+	exit(EXIT_FAILURE);
 }
 
 int main(int argc, char **argv)
@@ -207,28 +176,33 @@ int main(int argc, char **argv)
 	/* special cases since we're not using getopt */
 	if (!strcmp(argv[1], "-V")) {
 		printf("Network UPS Tools upsc %s\n", UPS_VERSION);
-		exit(0);
+		exit(EXIT_SUCCESS);
 	}
 
 	if (!strcmp(argv[1], "-h"))
 		help(argv[0]);
 
-	upscli_splitname(argv[1], &upsname, &hostname, &port);
+	check_upsdef(argv[1]);
+
+	upsname = hostname = NULL;
+
+	if (upscli_splitname(argv[1], &upsname, &hostname, &port) != 0)
+		clean_exit(&ups, upsname, hostname, EXIT_FAILURE);
 
 	if (upscli_connect(&ups, hostname, port, UPSCLI_CONN_TRYSSL) < 0) {
 		fprintf(stderr, "Error: %s\n", upscli_strerror(&ups));
 
-		clean_exit(&ups, upsname, hostname, 1);
+		clean_exit(&ups, upsname, hostname, EXIT_FAILURE);
 	}
 
 	if (argc >= 3) {
-		ret = new_printvar(&ups, upsname, argv[2]);
+		ret = printvar(&ups, upsname, argv[2]);
 		clean_exit(&ups, upsname, hostname, ret);
 	}
 
-	ret = new_list(&ups, upsname);
+	ret = list_vars(&ups, upsname);
 	clean_exit(&ups, upsname, hostname, ret);
 
 	/* NOTREACHED */
-	return 0;
+	exit(EXIT_FAILURE);
 }
