@@ -58,9 +58,9 @@ static	int	port;
 static	char	*upsname, *hostname;
 static	UPSCONN	ups;
 
-#define RED(x)         ((x >> 16) & 0xff)
-#define GREEN(x)       ((x >> 8)  & 0xff)
-#define BLUE(x)        (x & 0xff)
+#define RED(x)		((x >> 16) & 0xff)
+#define GREEN(x)	((x >> 8)  & 0xff)
+#define BLUE(x)		(x & 0xff)
 
 
 void parsearg(char *var, char *value) 
@@ -130,7 +130,7 @@ static void drawimage(gdImagePtr im)
 
 	upscli_disconnect(&ups);
 
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 /* helper function to allocate color in the image */
@@ -197,7 +197,7 @@ static void drawscale(
 		if (level % step10 == 0) {
 			gdImageLine(im, 0, y, width, y, col1);
 		} else {
-		        if (level % step5 == 0)
+			if (level % step5 == 0)
 				gdImageLine(im, 5, y, width - 5, y, col2);
 			else
 		    		gdImageLine(im, 10, y, width - 10, y, col2);
@@ -208,9 +208,9 @@ static void drawscale(
 	for (level = lvlhi; level >= lvllo; level -= step) {
 		if (level % step10 == 0) {
 			y = scale_height * (lvlhi - level) / range;
-		        snprintf(lbltxt, sizeof(lbltxt), "%u", level);
+			snprintf(lbltxt, sizeof(lbltxt), "%u", level);
 			gdImageString(im, gdFontMediumBold, width - 20, y, 
-				lbltxt, scale_num_color);
+				(unsigned char *) lbltxt, scale_num_color);
 		}
 	}
 }
@@ -245,7 +245,7 @@ static void drawbar(
 		redlo2, redhi2, grnlo, grnhi);
 	
 	/* allocate colors for the bar and summary text */
-        bar_color	= color_alloc(im, get_imgarg("bar_col"));
+	bar_color	= color_alloc(im, get_imgarg("bar_col"));
 	summary_color	= color_alloc(im, get_imgarg("summary_col"));
 
 	/* rescale UPS value to fit in the scale */
@@ -269,7 +269,8 @@ static void drawbar(
 	snprintf(text, sizeof(text), format, value);
 	gdImageString(im, gdFontMediumBold, 
 		(width - strlen(text)*gdFontMediumBold->w)/2, 
-		height - gdFontMediumBold->h, text, summary_color);
+		height - gdFontMediumBold->h, 
+		(unsigned char *) text, summary_color);
 
 	drawimage(im);
 
@@ -277,11 +278,17 @@ static void drawbar(
 }
 
 /* draws the error image */
-static void noimage(const char *msg)
+static void noimage(const char *fmt, ...)
 {
 	gdImagePtr	im;
 	int		back_color, summary_color;
 	int		width, height;
+	char		msg[SMALLBUF];
+	va_list		ap;
+
+	va_start(ap, fmt);
+	vsnprintf(msg, sizeof(msg), fmt, ap);
+	va_end(ap);
 
 	width = get_imgarg("width");
 	height = get_imgarg("height");
@@ -296,13 +303,13 @@ static void noimage(const char *msg)
 	if (width > height)
 		gdImageString(im, gdFontMediumBold, 
 			(width - strlen(msg)*gdFontMediumBold->w)/2, 
-			(height - gdFontMediumBold->h)/2, (char*)msg, 
-			summary_color);
+			(height - gdFontMediumBold->h)/2, 
+			(unsigned char *) msg, 	summary_color);
 	else
 		gdImageStringUp(im, gdFontMediumBold, 
 			(width - gdFontMediumBold->h)/2, 
 			(height + strlen(msg)*gdFontMediumBold->w)/2, 
-			(char*)msg, summary_color);
+			(unsigned char *) msg, summary_color);
 
 	drawimage(im);
 
@@ -355,7 +362,7 @@ static void drawgeneralbar(double var, int min, int nom, int max,
 	/* NOTREACHED */
 }
 
-/* draws UTILITY and OUTVOLT bar style indicators */
+/* draws input and output voltage bar style indicators */
 static void draw_utility(double var, int min, int nom, int max, 
 		int deviation, const char *format)
 {
@@ -370,32 +377,57 @@ static void draw_utility(double var, int min, int nom, int max,
 	if (max == -1) 
 		max = (var < 200.0) ? 140 : 250;
 	
-	drawgeneralbar(var, min, nom, max, deviation, format);
+	drawgeneralbar(var, min, -1, max, deviation, format);
 
 	/* NOTREACHED */
 }
 
-/* draws BATTPCT bar style indicator */
+/* draws battery.percent bar style indicator */
 static void draw_battpct(double var, int min, int nom, int max, 
 		int deviation, const char *format)
 {
 	drawbar(0, 100, 1, 10, 20, 0, 50, -1, -1, 80, 100, var, format);
 }
 
-/* draws BATTVOLT bar style indicator */
+/* draws battery.voltage bar style indicator */
 static void draw_battvolt(double var, int min, int nom, int max, 
 		int deviation, const char *format)
 {
 	drawbar(10, 40, 1, 5, 10, 0, 20, 28, 40, 25, 27, var, format);
 }
 
-/* draws UPSLOAD bar style indicator */
+/* draws ups.load bar style indicator */
 static void draw_upsload(double var, int min, int nom, int max, 
 		int deviation, const char *format)
 {
 	drawbar(0, 125, 5, 5, 25, 100, 125, -1, -1, 0, 50, var, format);
 }
 
+static int get_var(const char *var, char *buf, size_t buflen)
+{
+	int	ret;
+	unsigned int	numq, numa;
+	const	char	*query[4];
+	char	**answer;
+
+	query[0] = "VAR";
+	query[1] = upsname;
+	query[2] = var;
+
+	numq = 3;
+
+	ret = upscli_get(&ups, numq, query, &numa, &answer);
+
+	if (ret < 0)
+		return 0;
+
+	if (numa < numq)
+		return 0;
+
+	snprintf(buf, buflen, "%s", answer[3]);
+	return 1;
+}
+
 int main(int argc, char **argv)
 {
 	char	str[SMALLBUF];
@@ -411,12 +443,17 @@ int main(int argc, char **argv)
 	if (!checkhost(monhost, NULL))
 		noimage("Access denied");
 
-	upscli_splitname(monhost, &upsname, &hostname, &port);
+	upsname = hostname = NULL;
+
+	if (upscli_splitname(monhost, &upsname, &hostname, &port) != 0) {
+		noimage("Invalid UPS definition (upsname@hostname)\n");
+		exit(EXIT_FAILURE);
+	}
 
 	if (upscli_connect(&ups, hostname, port, 0) < 0) {
-		printf("Can't connect to server: %s\n", 
+		noimage("Can't connect to server:\n%s\n", 
 			upscli_strerror(&ups));
-		exit(0);
+		exit(EXIT_FAILURE);
 	}
 	
 	for (i = 0; imgvar[i].name; i++)
@@ -426,19 +463,18 @@ int main(int argc, char **argv)
 			   registered with this variable */
 			if (!imgvar[i].drawfunc) {
 				noimage("Draw function N/A");
-				exit(1);
+				exit(EXIT_FAILURE);
 			}
 			
 			/* get the variable value */
-			if (upscli_getvar(&ups, upsname, imgvar[i].name, 
-				str, sizeof(str)) >= 0)
+			if (get_var(imgvar[i].name, str, sizeof(str)) == 1) {
 				var = strtod(str, NULL);
-			else {
+			} else {
 				/* no value, no fun */
 				snprintf(str, sizeof(str), "%s N/A", 
 					imgvar[i].name);
 				noimage(str);
-				exit(2);
+				exit(EXIT_FAILURE);
 			}
 			
 			/* when getting minimum, nominal and maximum values, 
@@ -448,9 +484,8 @@ int main(int argc, char **argv)
 	
 			/* get the minimum value */
 			if (imgvar[i].minimum) {
-				if (upscli_getvar(&ups, upsname, 
-					imgvar[i].minimum, str, 
-					sizeof(str)) >= 0) {
+				if (get_var(imgvar[i].minimum, str, 
+					sizeof(str)) == 1) {
 					min = atoi(str);
 				} else {
 					min = get_imgarg(imgvar[i].minimum);
@@ -462,9 +497,8 @@ int main(int argc, char **argv)
 	
 			/* get the nominal value */
 			if (imgvar[i].nominal) {
-				if (upscli_getvar(&ups, upsname, 
-					imgvar[i].nominal, str, 
-					sizeof(str)) >= 0) {
+				if (get_var(imgvar[i].nominal, str, 
+					sizeof(str)) == 1) {
 					nom = atoi(str);
 				} else {
 					nom = get_imgarg(imgvar[i].nominal);
@@ -476,9 +510,8 @@ int main(int argc, char **argv)
 	
 			/* get the maximum value */
 			if (imgvar[i].maximum) {
-				if (upscli_getvar(&ups, upsname, 
-					imgvar[i].maximum, str, 
-					sizeof(str)) >= 0) {
+				if (get_var(imgvar[i].maximum, str,
+					sizeof(str)) == 1) {
 					max = atoi(str);
 				} else {
 					max = get_imgarg(imgvar[i].maximum);
@@ -490,30 +523,41 @@ int main(int argc, char **argv)
 
 			imgvar[i].drawfunc(var, min, nom, max, 
 				imgvar[i].deviation, imgvar[i].format);
-			exit(0);
+			exit(EXIT_SUCCESS);
 		} 
 
 	noimage("Unknown display");
-	return 0;
+	exit(EXIT_FAILURE);
 }
 
 struct imgvar_t imgvar[] = {
-	{ "utility",	"lowxfer",	"nom_in_volt",	"highxfer",	10,
-		"%.1f VAC",	draw_utility },
-	{ "battpct",	NULL,		NULL,		NULL,		0,	
-		"%.1f %%",	draw_battpct },
-      { "battvolt",   NULL,           NULL,           NULL,           0,      
-              "%.1f VDC",     draw_battvolt },
-	{ "upstemp",	"tempmin",	NULL,		"tempmax",	10,	
-		"%.1f C",	drawgeneralbar },
-	{ "acfreq",	NULL,		"nom_in_freq",	NULL,		2,	
-		"%.1f Hz",	drawgeneralbar },
-	{ "loadpct",	NULL,		NULL,		NULL,		0,	
-		"%.1f %%",	draw_upsload },
-	{ "outvolt",	"lowxfer",	"nom_out_volt",	"highxfer",	10,	
-		"%.1f VAC",	draw_utility },
-	{ "out_freq",	NULL,		"nom_out_freq",	NULL,		2,	
-		"%.1f Hz",	drawgeneralbar },
+	{ "input.voltage", "input.transfer.low", "input.voltage.nominal",
+		"input.transfer.high", 10, 
+		"%.1f VAC", draw_utility				},
+
+	{ "battery.charge", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_battpct				},
+
+	{ "battery.voltage", NULL, NULL, NULL, 0,
+		"%.1f VDC",	draw_battvolt				},
+
+	{ "ups.temperature", "ups.temperature.minimum", NULL,
+		"ups.temperature.maximum", 10,
+		"%.1f C",	drawgeneralbar				},
+
+	{ "input.frequency", NULL, "input.frequency.nominal", NULL, 2,
+		"%.1f Hz",	drawgeneralbar				},
+
+	{ "ups.load", NULL, NULL, NULL, 0,
+		"%.1f %%",	draw_upsload				},
+
+	{ "output.voltage", "input.transfer.low", "output.voltage.nominal",
+		"input.transfer.high", 10, 
+		"%.1f VAC",	draw_utility				},
+
+	{ "output.frequency", NULL, "output.frequency.nominal", NULL, 2,
+		"%.1f Hz",	drawgeneralbar				},
+
 	{ NULL,		NULL,		NULL,		NULL,		0,	
 		NULL,		NULL }
 };
