@@ -44,7 +44,7 @@
 
 	static	FILE	*logfile;
 	static	const	char *logfn, *monhost;
-	static	sigset_t	upsl_sigmask;
+	static	sigset_t	nut_upslog_sigmask;
 	static	char	logbuffer[LARGEBUF], *logformat;
 
 	static	struct	flist_t	*fhead = NULL;
@@ -81,9 +81,9 @@ static void setup_signals(void)
 {
 	struct	sigaction	sa;
 
-	sigemptyset(&upsl_sigmask);
-	sigaddset(&upsl_sigmask, SIGHUP);
-	sa.sa_mask = upsl_sigmask;
+	sigemptyset(&nut_upslog_sigmask);
+	sigaddset(&nut_upslog_sigmask, SIGHUP);
+	sa.sa_mask = nut_upslog_sigmask;
 	sa.sa_handler = set_reopen_flag;
 	sa.sa_flags = 0;
 	if (sigaction(SIGHUP, &sa, NULL) < 0)
@@ -153,7 +153,7 @@ static void do_upshost(const char *arg)
 
 static void do_pid(const char *arg)
 {
-	snprintfcat(logbuffer, sizeof(logbuffer), "%d", getpid());
+	snprintfcat(logbuffer, sizeof(logbuffer), "%ld", (long)getpid());
 }
 
 static void do_time(const char *arg)
