@@ -2,6 +2,7 @@
 #ifndef HAVE_SETENV
 #include <stdlib.h>
 #include <string.h>
+#include "common.h"
 
 int setenv(const char *name, const char *value, int overwrite)
 {
@@ -15,7 +16,7 @@ int setenv(const char *name, const char *value, int overwrite)
 			return 0;
 		}
 	}
-	buffer = malloc(strlen(value) + strlen(name) + 2);
+	buffer = xmalloc(strlen(value) + strlen(name) + 2);
 	strcpy(buffer, name);
 	strcat(buffer, "=");
 	strcat(buffer, value);
