@@ -899,7 +899,7 @@ void upsdrv_initinfo(void)
 	val = belkin_nut_read_int(REG_FIRMWARE);
 	if (val!=-1) {
 		dstate_setinfo("ups.firmware", "%d", (val>>4) & 0xf);
-		dstate_setinfo("ups.type", upstype[(val & 0x0f) % 3]);
+		dstate_setinfo("ups.type", "%s", upstype[(val & 0x0f) % 3]);
 	}
 
 	/* read writable values and declare them writable */
