@@ -2,6 +2,8 @@
 
    Copyright (C) 1999  Russell Kroll <rkroll@exploits.org>
 
+   ID config option by Jason White <jdwhite@jdwhite.org>
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
@@ -27,7 +29,7 @@
 #define SER_WAIT_SEC	3	/* allow 3.0 sec for ser_get calls */
 #define SER_WAIT_USEC	0
 
-#define DRV_VERSION "1.02"
+#define DRV_VERSION "1.03"
 
 static	float	lowvolt = 0, highvolt = 0, voltrange = 0;
 static	int	linenorm = 0;
@@ -73,6 +75,12 @@ static void model_set(const char *abbr, const char *rating)
 		return;
 	}
 
+	if (!strcmp(abbr, "610")) {
+		dstate_setinfo("ups.mfr", "%s", "Best Power");
+		dstate_setinfo("ups.model", "610 %s", rating);
+		return;
+	}
+
 	if (!strcmp(abbr, "620")) {
 		dstate_setinfo("ups.mfr", "%s", "Sola Australia");
 		dstate_setinfo("ups.model", "Sola 620 %s", rating);
@@ -107,10 +115,18 @@ static int instcmd(const char *cmdname, const char *extra)
 	return STAT_INSTCMD_UNKNOWN;
 }
 
-
 static int get_ident(char *buf, size_t bufsize)
 {
 	int	i, ret;
+	char	*ID;
+
+	ID = getval("ID");	/* user-supplied override from ups.conf */
+
+	if (ID) {
+		upsdebugx(2, "NOTE: using user-supplied ID response");
+		snprintf(buf, bufsize, "%s", ID);
+		return 1;
+	}
 
 	for (i = 0; i < MAXTRIES; i++) {
 		ser_send_pace(upsfd, UPSDELAY, "\rID\r");
@@ -374,6 +390,7 @@ void upsdrv_help(void)
 void upsdrv_makevartable(void)
 {
 	addvar(VAR_VALUE, "nombattvolt", "Override nominal battery voltage");
+	addvar(VAR_VALUE, "ID", "Force UPS ID response string");
 }
 
 void upsdrv_banner(void)
