@@ -149,10 +149,9 @@ static int scan_poll_values(char *buf)
 	double rminutes;
 
 /*	These are used to hold status of UPS.
- *	val1 = online/onbattery status
+ *	tmp1 = online/onbattery status
  */
- 	char temp1=values[6][0];
-	char *tmp1=&temp1;
+	char *tmp1 = &buf[pollstatusmap[POLL_UPSSTATUS].begin];
 
 	while ((pollstatusmap[i].end != 0))
 	{
