@@ -28,6 +28,8 @@
 #ifndef _LIBHID_H
 #define _LIBHID_H
 
+#include "config.h"
+
 #include <sys/types.h>
 #include <regex.h>
 #include "timehead.h"
@@ -41,6 +43,10 @@ typedef enum ebool { FALSE, TRUE } bool;
 typedef int bool;
 #endif
 
+#ifdef HAVE_STDINT_H
+#include <stdint.h> /* for uint16_t */
+#endif
+
 /* Device open modes */
 #define MODE_OPEN 0
 #define MODE_REOPEN 1
@@ -127,13 +133,6 @@ void free_regex_matcher(HIDDeviceMatcher_t *matcher);
 {
 	char*   Path;			/*!< HID Object's fully qualified HID path	*/
 	long    Value;		/*!< HID Object Value					*/
-	u_char   Attribute;	/*!< Report field attribute				*/
-	u_long   Unit; 		/*!< HID Unit							*/
-	char    UnitExp;		/*!< Unit exponent						*/
-	long    LogMin;		/*!< Logical Min							*/
-	long    LogMax;		/*!< Logical Max						*/
-	long    PhyMin;		/*!< Physical Min						*/
-	long    PhyMax;		/*!< Physical Max						*/	
 } HIDItem;
 
 /* Describe a set of values to match for finding a special HID device.
@@ -167,7 +166,7 @@ HIDItem *HIDGetItem(const char *ItemPath);
 /*
  * HIDGetItemValue
  * -------------------------------------------------------------------------- */
-float HIDGetItemValue(usb_dev_handle *udev, char *path, float *Value, usage_tables_t *utab);
+int HIDGetItemValue(usb_dev_handle *udev, char *path, float *Value, usage_tables_t *utab);
 
 /*
  * HIDGetItemString
