@@ -90,8 +90,8 @@ int libusb_open(usb_dev_handle **udevp, HIDDevice *curDevice, HIDDeviceMatcher_t
 #endif
 	struct my_usb_hid_descriptor *desc;
 	HIDDeviceMatcher_t *m;
-	struct usb_device *dev;                                                
-	struct usb_bus *bus;                                                   
+	struct usb_device *dev;
+	struct usb_bus *bus;
 	usb_dev_handle *udev;
 	
 	int ret, res; 
@@ -231,6 +231,9 @@ int libusb_open(usb_dev_handle **udevp, HIDDevice *curDevice, HIDDeviceMatcher_t
 			res = usb_control_msg(udev, USB_ENDPOINT_IN+1, USB_REQ_GET_DESCRIPTOR,
 					      (USB_DT_REPORT << 8) + 0, 0, ReportDesc, 
 					      desc->wDescriptorLength, USB_TIMEOUT);
+			
+			TRACE(2, "Size read for the report descriptor: %u", res);
+
 			if (res >= desc->wDescriptorLength) 
 			{
 				TRACE(2, "Report descriptor retrieved (Reportlen = %u)", desc->wDescriptorLength);
@@ -314,7 +317,10 @@ int libusb_get_string(usb_dev_handle *udev, int StringIdx, char *string)
 int libusb_get_interrupt(usb_dev_handle *udev, unsigned char *buf, int bufsize, int timeout)
 {
   int ret = -1;
-
+#if SUN_LIBUSB
+	/* sleep during timeout to slow down a bit... */
+	sleep(timeout / 1000);
+#else
   if (udev != NULL)
 	{
 	  /* FIXME: hardcoded interrupt EP => need to get EP descr for IF descr */
@@ -324,7 +330,7 @@ int libusb_get_interrupt(usb_dev_handle *udev, unsigned char *buf, int bufsize,
 	  else
 		TRACE(6, " none (%i)", ret);
 	}
-
+#endif
   return ret;
 }
 
@@ -332,9 +338,9 @@ void libusb_close(usb_dev_handle *udev)
 {
 	if (udev != NULL)
 	{
-	        /* usb_release_interface() sometimes blocks and goes
-	           into uninterruptible sleep.  So don't do it. */
-	        /* usb_release_interface(udev, 0); */
+		/* usb_release_interface() sometimes blocks and goes
+		into uninterruptible sleep.  So don't do it. */
+		/* usb_release_interface(udev, 0); */
 		usb_close(udev);
 	}
 }
