@@ -7,7 +7,7 @@
  *    This model is based on PowerCom (www.powercom.com) models.
  *  -Socomec Sicon Egys 420
  *
- * $Id: powercom.c,v 1.1.1.1 2005/01/27 14:33:20 aquette Exp $
+ * $Id: powercom.c 440 2006-07-10 12:33:14Z aquette $
  *
  * Copyrights:
  * (C) 2002 Simon Rozman <simon@rozman.net>
@@ -124,7 +124,7 @@ static struct type types[] = {
  * local used functions
  */
 
-static void shutdown(void)
+static void nut_shutdown(void)
 {
 	printf ("Initiating UPS shutdown!\n");
         
@@ -354,7 +354,7 @@ void upsdrv_shutdown(void)
 {
 	/* power down the attached load immediately */
 	printf("Forced UPS shutdown triggered, do it...\n");
-	shutdown();
+	nut_shutdown();
 }
 
 /* initialize UPS */
