@@ -41,8 +41,8 @@
 #define IDENT_MAXTRIES   5
 #define IDENT_MINSUCCESS 2
 
-#define SEND_PACE    50000 /* 50ms interval between chars */
-#define READ_TIMEOUT 2     /* 2 seconds timeout on read */
+#define SEND_PACE    100000 /* 100ms interval between chars */
+#define READ_TIMEOUT 2      /* 2 seconds timeout on read */
 
 #define MAX_START_DELAY    9999
 #define MAX_SHUTDOWN_DELAY 99
@@ -275,7 +275,7 @@ void upsdrv_updateinfo(void)
 	dstate_setinfo("input.voltage.fault", "%.1f", query.fvolt);
 	dstate_setinfo("output.voltage", "%.1f", query.ovolt);
 	dstate_setinfo("ups.load", "%.1f", query.load);
-	dstate_setinfo("output.frequency", "%.1f", query.freq);
+	dstate_setinfo("input.frequency", "%.1f", query.freq);
 	dstate_setinfo("battery.voltage", "%.1f", query.battvolt);
 
 	/* this value seems to be bogus, it always reports 37.8 */
