@@ -45,8 +45,8 @@
 */
 
 struct {
-	char	*mtext;
-	char	*desc;
+	const	char	*mtext;
+	const	char	*desc;
 	float	lowvolt;
 	float	voltrange;
 	int	lowxfer;
