@@ -267,7 +267,7 @@ static void load_upsdconf(int reloading)
 				snprintfcat(errmsg, sizeof(errmsg), " %s", 
 					ctx.arglist[i]);
 
-			upslogx(LOG_WARNING, errmsg);
+			upslogx(LOG_WARNING, "%s", errmsg);
 		}
 
 	}
