@@ -25,13 +25,16 @@
 #include <openssl/ssl.h>
 #endif
 
+#include "parseconf.h"
+
 /* client structure */
 typedef struct {
 	char	*addr;
 	int	fd;
+	int	delete;			/* set after a write fails */
 	struct sockaddr_in sock;
 	char	rq[SMALLBUF];
-	int	rqpos;
+	size_t	rqpos;
 	char	*loginups;
 	char	*password;
 	char	*username;
@@ -43,6 +46,8 @@ typedef struct {
 #endif
 	int	ssl_connected;
 
+	PCONF_CTX	ctx;
+
 	void	*next;
 }	ctype;
 
