@@ -27,6 +27,8 @@
 #ifndef UPSD_H_SEEN
 #define UPSD_H_SEEN
 
+#include "attribute.h"
+
 #include "common.h"
 
 #include <sys/socket.h>
@@ -50,7 +52,8 @@ upstype *get_ups_ptr(const char *upsname);
 int ups_available(const upstype *ups, ctype *client);
 
 void kick_login_clients(const char *upsname);
-int sendback(ctype *client, const char *fmt, ...);
+int sendback(ctype *client, const char *fmt, ...)
+	__attribute__ ((__format__ (__printf__, 2, 3)));
 int send_err(ctype *client, const char *errtype);
 
 void check_perms(const char *fn);
